@@ -4,7 +4,7 @@
     ~~~~~~~~~~~~~~~~~~~~~~
 
 
-    :copyright: (c) 2010 by the Werkzeug Team, see AUTHORS for more details.
+    :copyright: (c) 2011 by the Werkzeug Team, see AUTHORS for more details.
     :license: BSD license.
 """
 import pickle
@@ -54,7 +54,7 @@ def prepare_environ_pickle(environ):
     for key, value in environ.iteritems():
         try:
             pickle.dumps((key, value))
-        except:
+        except Exception:
             continue
         result[key] = value
     return result
@@ -94,6 +94,15 @@ def test_base_request():
     ## assert response['form_as_list'] == [('foo', ['blub hehe']), ('blah', ['42'])]
     assert_environ(response['environ'], 'POST')
 
+    # patch requests with form data
+    response = client.patch('/?blub=blah', data='foo=blub+hehe&blah=42',
+                            content_type='application/x-www-form-urlencoded')
+    assert response['args'] == MultiDict([('blub', 'blah')])
+    assert response['args_as_list'] == [('blub', ['blah'])]
+    assert response['form'] == MultiDict([('foo', 'blub hehe'), ('blah', '42')])
+    assert response['data'] == ''
+    assert_environ(response['environ'], 'PATCH')
+
     # post requests with json data
     json = '{"foo": "bar", "blub": "blah"}'
     response = client.post('/?a=b', data=json, content_type='application/json')
@@ -108,10 +117,6 @@ def test_access_route():
         'X-Forwarded-For': '192.168.1.2, 192.168.1.1'
     })
     req.environ['REMOTE_ADDR'] = '192.168.1.3'
-    req.is_behind_proxy = True
-    assert req.access_route == ['192.168.1.2', '192.168.1.1']
-    assert req.remote_addr == '192.168.1.2'
-    req.is_behind_proxy = False
     assert req.access_route == ['192.168.1.2', '192.168.1.1']
     assert req.remote_addr == '192.168.1.3'
 
@@ -130,6 +135,10 @@ def test_url_request_descriptors():
     assert req.url_root == 'http://example.com/test/'
     assert req.host_url == 'http://example.com/'
     assert req.host == 'example.com'
+    assert req.scheme == 'http'
+
+    req = Request.from_values('/bar?foo=baz', 'https://example.com/test')
+    assert req.scheme == 'https'
 
 
 def test_authorization_mixin():
@@ -319,12 +328,14 @@ def test_etag_response_mixin():
     response.headers['Content-Length'] = len(response.data)
     assert response.headers['Cache-Control'] == 'must-revalidate, max-age=60'
 
+    assert 'date' not in response.headers
     env = create_environ()
     env.update({
         'REQUEST_METHOD':       'GET',
         'HTTP_IF_NONE_MATCH':   response.get_etag()[0]
     })
     response.make_conditional(env)
+    assert 'date' in response.headers
 
     # after the thing is invoked by the server as wsgi application
     # (we're emulating this here), there must not be any entity
@@ -333,6 +344,13 @@ def test_etag_response_mixin():
     assert resp.status_code == 304
     assert not 'content-length' in resp.headers
 
+    # make sure date is not overriden
+    response = Response('Hello World')
+    response.date = 1337
+    d = response.date
+    response.make_conditional(env)
+    assert response.date == d
+
 
 def test_etag_response_mixin_freezing():
     """Freeze of the etag response mixin adds etag if mixed first"""
@@ -556,7 +574,7 @@ def test_new_response_iterator_behavior():
 
     # automatic generator sequence conversion
     resp.response = generate_items()
-    resp.implicit_seqence_conversion = False
+    resp.implicit_sequence_conversion = False
     assert resp.is_streamed
     assert not resp.is_sequence
     assert_raises(RuntimeError, lambda: resp.data)
@@ -568,7 +586,7 @@ def test_new_response_iterator_behavior():
 
     # stream makes it a list no matter how the conversion is set
     for val in True, False:
-        resp.implicit_seqence_conversion = val
+        resp.implicit_sequence_conversion = val
         resp.response = ("foo", "bar")
         assert resp.is_sequence
         resp.stream.write('baz')
@@ -627,3 +645,36 @@ def test_response_headers_passthrough():
     headers = Headers()
     resp = Response(headers=headers)
     assert resp.headers is headers
+
+
+def test_response_304_no_content_length():
+    """Ensure that 304 responses don't get an automatic content length"""
+    resp = Response('Test', status=304)
+    env = create_environ()
+    assert 'content-length' not in resp.get_wsgi_headers(env)
+
+
+def test_ranges():
+    """Test basic range support"""
+    # basic range stuff
+    req = Request.from_values()
+    assert req.range is None
+    req = Request.from_values(headers={'Range': 'bytes=0-499'})
+    assert req.range.ranges == [(0, 500)]
+
+    resp = Response()
+    resp.content_range = req.range.make_content_range(1000)
+    assert resp.content_range.units == 'bytes'
+    assert resp.content_range.start == 0
+    assert resp.content_range.stop == 500
+    assert resp.content_range.length == 1000
+    assert resp.headers['Content-Range'] == 'bytes 0-499/1000'
+
+    resp.content_range.unset()
+    assert 'Content-Range' not in resp.headers
+
+    resp.headers['Content-Range'] = 'bytes 0-499/1000'
+    assert resp.content_range.units == 'bytes'
+    assert resp.content_range.start == 0
+    assert resp.content_range.stop == 500
+    assert resp.content_range.length == 1000
